@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Applicator;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Applicator vocabulary
 
-our $VERSION = '0.547';
+our $VERSION = '0.548';
 
 use 5.020;
 use Moo;
@@ -83,10 +83,8 @@ sub _eval_keyword_allOf ($self, $data, $schema, $state) {
     }
   }
 
-  if (@invalid == 0) {
-    push $state->{annotations}->@*, @new_annotations;
-    return 1;
-  }
+  push $state->{annotations}->@*, @new_annotations;
+  return 1 if @invalid == 0;
 
   my $pl = @invalid > 1;
   return E($state, 'subschema%s %s %s not valid', $pl?'s':'', join(', ', @invalid), $pl?'are':'is');
@@ -125,10 +123,9 @@ sub _eval_keyword_oneOf ($self, $data, $schema, $state) {
     last if @valid > 1 and $state->{short_circuit};
   }
 
-  if (@valid == 1) {
-    push $state->{annotations}->@*, @new_annotations;
-    return 1;
-  }
+  push $state->{annotations}->@*, @new_annotations;
+  return 1 if @valid == 1;
+
   if (not @valid) {
     push $state->{errors}->@*, @errors;
     return E($state, 'no subschemas are valid');
@@ -144,7 +141,7 @@ sub _eval_keyword_not ($self, $data, $schema, $state) {
   return 1 if not $self->eval($data, $schema->{not},
     +{ %$state, schema_path => $state->{schema_path}.'/not',
       short_circuit => $state->{short_circuit} || !$state->{collect_annotations},
-      errors => [], annotations => [ $state->{annotations}->@* ] });
+      errors => [] });
 
   return E($state, 'subschema is valid');
 }
@@ -192,8 +189,8 @@ sub _eval_keyword_dependentSchemas ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  return E($state, 'not all dependencies are satisfied') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
+  return E($state, 'not all dependencies are satisfied') if not $valid;
   return 1;
 }
 
@@ -252,8 +249,8 @@ sub _eval_keyword_dependencies ($self, $data, $schema, $state) {
     }
   }
 
-  return E($state, 'not all dependencies are satisfied') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
+  return E($state, 'not all dependencies are satisfied') if not $valid;
   return 1;
 }
 
@@ -276,7 +273,6 @@ sub _eval_keyword_items ($self, $data, $schema, $state) {
   return $self->_eval_keyword__items_array_schemas($data, $schema, $state)
     if is_plain_arrayref($schema->{items});
 
-  $state->{_last_items_index} //= -1;
   return $self->_eval_keyword__items_schema($data, $schema, $state);
 }
 
@@ -290,6 +286,7 @@ sub _eval_keyword_additionalItems ($self, $data, $schema, $state) {
 # prefixItems (draft 2020-12), array-based items (all drafts)
 sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
+  return 1 if ($state->{_last_items_index}//-1) == $data->$#*;
 
   my @orig_annotations = $state->{annotations}->@*;
   my @new_annotations;
@@ -320,22 +317,22 @@ sub _eval_keyword__items_array_schemas ($self, $data, $schema, $state) {
     };
   }
 
-  return E($state, 'not all items are valid') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
-  return A($state,
-    ($state->{_last_items_index}//-1) == $data->$#* ? true : $state->{_last_items_index});
+  A($state, $state->{_last_items_index} == $data->$#* ? true : $state->{_last_items_index});
+  return E($state, 'not all items are valid') if not $valid;
+  return 1;
 }
 
 # schema-based items (all drafts), and additionalItems (up to and including draft2019-09)
 sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
-  return 1 if $state->{_last_items_index} == $data->$#*;
+  return 1 if ($state->{_last_items_index}//-1) == $data->$#*;
 
   my @orig_annotations = $state->{annotations}->@*;
   my @new_annotations;
   my $valid = 1;
 
-  foreach my $idx ($state->{_last_items_index}+1 .. $data->$#*) {
+  foreach my $idx (($state->{_last_items_index}//-1)+1 .. $data->$#*) {
     if (is_type('boolean', $schema->{$state->{keyword}})) {
       next if $schema->{$state->{keyword}};
       $valid = E({ %$state, data_path => $state->{data_path}.'/'.$idx },
@@ -359,10 +356,11 @@ sub _eval_keyword__items_schema ($self, $data, $schema, $state) {
 
   $state->{_last_items_index} = $data->$#*;
 
+  push $state->{annotations}->@*, @new_annotations;
+  A($state, true);
   return E($state, 'subschema is not valid against all %sitems',
     $state->{keyword} eq 'additionalItems' ? 'additional ' : '') if not $valid;
-  push $state->{annotations}->@*, @new_annotations;
-  return A($state, true);
+  return 1;
 }
 
 sub _traverse_keyword_contains { shift->traverse_subschema(@_) }
@@ -408,16 +406,13 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
 
   my $valid = 1;
   my @orig_annotations = $state->{annotations}->@*;
-  my (@valid_properties, @new_annotations);
+  my (@properties, @new_annotations);
   foreach my $property (sort keys $schema->{properties}->%*) {
     next if not exists $data->{$property};
+    push @properties, $property;
 
     if (is_type('boolean', $schema->{properties}{$property})) {
-      if ($schema->{properties}{$property}) {
-        push @valid_properties, $property;
-        next;
-      }
-
+      next if $schema->{properties}{$property};
       $valid = E({ %$state, data_path => jsonp($state->{data_path}, $property),
         _schema_path_suffix => $property }, 'property not permitted');
     }
@@ -427,7 +422,6 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
           +{ %$state, annotations => \@annotations,
             data_path => jsonp($state->{data_path}, $property),
             schema_path => jsonp($state->{schema_path}, 'properties', $property) })) {
-        push @valid_properties, $property;
         push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
         next;
       }
@@ -437,9 +431,10 @@ sub _eval_keyword_properties ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  return E($state, 'not all properties are valid') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
-  return A($state, \@valid_properties);
+  A($state, \@properties);
+  return E($state, 'not all properties are valid') if not $valid;
+  return 1;
 }
 
 sub _traverse_keyword_patternProperties ($self, $schema, $state) {
@@ -458,15 +453,12 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
 
   my $valid = 1;
   my @orig_annotations = $state->{annotations}->@*;
-  my (@valid_properties, @new_annotations);
+  my (@properties, @new_annotations);
   foreach my $property_pattern (sort keys $schema->{patternProperties}->%*) {
     foreach my $property (sort grep m/$property_pattern/, keys %$data) {
+      push @properties, $property;
       if (is_type('boolean', $schema->{patternProperties}{$property_pattern})) {
-        if ($schema->{patternProperties}{$property_pattern}) {
-          push @valid_properties, $property;
-          next;
-        }
-
+        next if $schema->{patternProperties}{$property_pattern};
         $valid = E({ %$state, data_path => jsonp($state->{data_path}, $property),
           _schema_path_suffix => $property_pattern }, 'property not permitted');
       }
@@ -476,7 +468,6 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
             +{ %$state, annotations => \@annotations,
               data_path => jsonp($state->{data_path}, $property),
               schema_path => jsonp($state->{schema_path}, 'patternProperties', $property_pattern) })) {
-          push @valid_properties, $property;
           push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
           next;
         }
@@ -487,9 +478,10 @@ sub _eval_keyword_patternProperties ($self, $data, $schema, $state) {
     }
   }
 
-  return E($state, 'not all properties are valid') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
-  return A($state, [ uniqstr @valid_properties ]);
+  A($state, [ uniqstr @properties ]);
+  return E($state, 'not all properties are valid') if not $valid;
+  return 1;
 }
 
 sub _traverse_keyword_additionalProperties { shift->traverse_subschema(@_) }
@@ -499,18 +491,15 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
 
   my $valid = 1;
   my @orig_annotations = $state->{annotations}->@*;
-  my (@valid_properties, @new_annotations);
+  my (@properties, @new_annotations);
   foreach my $property (sort keys %$data) {
     next if exists $schema->{properties} and exists $schema->{properties}{$property};
     next if exists $schema->{patternProperties}
       and any { $property =~ /$_/ } keys $schema->{patternProperties}->%*;
 
+    push @properties, $property;
     if (is_type('boolean', $schema->{additionalProperties})) {
-      if ($schema->{additionalProperties}) {
-        push @valid_properties, $property;
-        next;
-      }
-
+      next if $schema->{additionalProperties};
       $valid = E({ %$state, data_path => jsonp($state->{data_path}, $property) },
         'additional property not permitted');
     }
@@ -520,7 +509,6 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
           +{ %$state, annotations => \@annotations,
             data_path => jsonp($state->{data_path}, $property),
             schema_path => $state->{schema_path}.'/additionalProperties' })) {
-        push @valid_properties, $property;
         push @new_annotations, @annotations[$#orig_annotations+1 .. $#annotations];
         next;
       }
@@ -530,9 +518,10 @@ sub _eval_keyword_additionalProperties ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  return E($state, 'not all additional properties are valid') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
-  return A($state, \@valid_properties);
+  A($state, \@properties);
+  return E($state, 'not all additional properties are valid') if not $valid;
+  return 1;
 }
 
 sub _traverse_keyword_propertyNames { shift->traverse_subschema(@_) }
@@ -557,8 +546,8 @@ sub _eval_keyword_propertyNames ($self, $data, $schema, $state) {
     last if $state->{short_circuit};
   }
 
-  return E($state, 'not all property names are valid') if not $valid;
   push $state->{annotations}->@*, @new_annotations;
+  return E($state, 'not all property names are valid') if not $valid;
   return 1;
 }
 
@@ -576,7 +565,7 @@ JSON::Schema::Modern::Vocabulary::Applicator - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.547
+version 0.548
 
 =head1 DESCRIPTION
 
